@@ -1,6 +1,4 @@
 /*
-  $Header: /cvs/src/mairix/rfc822.c,v 1.18 2004/01/11 23:46:54 richard Exp $
-
   mairix - message index builder and finder for maildir folders.
 
  **********************************************************************
@@ -98,12 +96,12 @@ static void splice_header_lines(struct line *header)/*{{{*/
     printf("header=<%s>\n", x->text);
 #endif
     next = x->next;
-    if (isspace(x->text[0])) {
+    if (isspace(x->text[0] & 0xff)) {
       /* Glue to previous line */
       char *p, *newbuf, *oldbuf;
       struct line *y;
       for (p=x->text; *p; p++) {
-        if (!isspace(*p)) break;
+        if (!isspace(*(unsigned char *)p)) break;
       }
       p--; /* point to final space */
       y = x->prev;
@@ -135,9 +133,9 @@ static int audit_header(struct line *header)/*{{{*/
   int first=1;
   int count=1;
   for (x=header->next; x!=header; x=x->next) {
-    int has_leading_space;
+    int has_leading_space=0;
     int is_blank;
-    int has_word_colon;
+    int has_word_colon=0;
 
     if (first) {
       /* Ignore any UUCP or mbox style From line at the start */
@@ -154,14 +152,14 @@ static int audit_header(struct line *header)/*{{{*/
     if (!is_blank) {
       char *p;
       int saw_char = 0;
-      has_leading_space = isspace(x->text[0]);
+      has_leading_space = isspace(x->text[0] & 0xff);
       has_word_colon = 0; /* default */
       p = x->text;
       while(*p) {
         if(*p == ':') {
           has_word_colon = saw_char;
           break;
-        } else if (isspace(*p)) {
+        } else if (isspace(*(unsigned char *) p)) {
           has_word_colon = 0;
           break;
         } else {
@@ -359,7 +357,7 @@ static enum encoding_type decode_encoding_type(char *e)/*{{{*/
   if (!e) {
     result = ENC_NONE;
   } else {
-    for (p=e; *p && isspace(*p); p++) ;
+    for (p=e; *p && isspace(*(unsigned char *)p); p++) ;
     if (   match_string("7bit", p)
         || match_string("7-bit", p)
         || match_string("7 bit", p)) {
@@ -403,7 +401,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
   result->boundary = NULL;
     
   p = hdrline;
-  while (*p && isspace(*p)) p++;
+  while (*p && isspace(*(unsigned char *)p)) p++;
   for (q=p+1; *q && (*q != '/'); q++) ;
 /*  assert(*q); */
   if (*q)
@@ -413,7 +411,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
     *s = 0;
    
     p = q + 1;
-    for (q=p+1; *q && !isspace(*q) && (*q != ';'); q++) ;
+    for (q=p+1; *q && !isspace(*(unsigned char *) q) && (*q != ';'); q++) ;
     result->minor = new_array(char, 1 + (q - p));
     for (s=result->minor; p<q;) *s++ = *p++;
     *s = 0;
@@ -426,7 +424,7 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
     while (semi && *semi) {
 
       name = semi + 1;
-      while (*name && isspace(*name)) name++;
+      while (*name && isspace(*(unsigned char *) name)) name++;
       if (!*name) break;
 
       for (eq=name+1; *eq && (*eq != '='); eq++) ;
@@ -434,7 +432,10 @@ static void parse_content_type(char *hdrline, struct content_type_header *result
       value = eq + 1;
       if (!*value) break;
 
-      for (semi = value+1; *semi && !isspace(*semi) && (*semi != ';'); semi++) ;
+      /* Find next semicolon, or end of line, or whitespace (if not a quoted RHS) */
+      for (semi = value+1; 
+           *semi && ((*value == '"') || (!isspace(*(unsigned char *)semi))) && (*semi != ';');
+           semi++) {}
 
       if (!strncasecmp(name, "boundary", 8)) {
         result->boundary = copy_string_start_end_unquote(value, semi);
@@ -466,7 +467,7 @@ static char *looking_at_ws_then_newline(char *start)/*{{{*/
   result = start;
   do {
          if (*result == '\n')   return result;
-    else if (!isspace(*result)) return NULL;
+    else if (!isspace(*(unsigned char *) result)) return NULL;
     else                        result++;
   } while (1);
 
@@ -586,7 +587,7 @@ static int split_and_splice_header(char *data, struct line *header, char **body_
     blank_line = 1; /* until proven otherwise */
     eol = sol;
     while (*eol && (*eol != '\n')) {
-      if (!isspace(*eol)) blank_line = 0;
+      if (!isspace(*(unsigned char *) eol)) blank_line = 0;
       eol++;
     }
     if (*eol == '\n') {
@@ -710,7 +711,7 @@ static void do_attachment(char *start, char *after_end, struct attachment *atts)
   if (body_start > after_end) {
     /* This is a (maliciously?) b0rken attachment, e.g. maybe empty */
     if (verbose) {
-      fprintf(stderr, "This message contains an invalid attachment, length=%d bytes\n", after_end - start);
+      fprintf(stderr, "This message contains an invalid attachment, length=%d bytes\n", (int)(after_end - start));
     }
   } else {
     body_len = after_end - body_start;
@@ -756,8 +757,8 @@ static void do_multipart(char *input, int input_len, char *boundary, struct atta
   /* Scan input to look for boundary markers */
   be = strstr(input, end_boundary);
   if (!be) {
-    fprintf(stderr, "Can't process multipart message without end boundary!\n");
-    goto cleanup;
+    fprintf(stderr, "Can't find end boundary in multipart message!\n");
+    be = strchr(input, 0); /* tolerate missing end boundary */
   }
 
   line_after_b0 = input;
@@ -770,8 +771,13 @@ static void do_multipart(char *input, int input_len, char *boundary, struct atta
       b1 = strstr(start_b1_search_from, normal_boundary);
 
       if (!b1) {
-        fprintf(stderr, "Oops, didn't find another normal boundary?\n");
-        goto cleanup;
+        if (*be) {
+          fprintf(stderr, "Oops, didn't find another normal boundary?\n");
+          goto cleanup;
+        } else {
+          b1 = be; /* tolerate missing end boundary */
+          break;
+        }
       }
       
       looking_at_end_boundary = (b1 == be);
@@ -918,7 +924,7 @@ struct rfc822 *data_to_rfc822(char *data, int length)/*{{{*/
   
 }
 /*}}}*/
-void create_ro_mapping(const char *filename, unsigned char **data, size_t *len)/*{{{*/
+void create_ro_mapping(const char *filename, unsigned char **data, int *len)/*{{{*/
 {
   struct stat sb;
   int fd;
@@ -960,7 +966,7 @@ void create_ro_mapping(const char *filename, unsigned char **data, size_t *len)/
 /*}}}*/
 struct rfc822 *make_rfc822(char *filename)/*{{{*/
 {
-  size_t len;
+  int len;
   char *data;
   struct rfc822 *result;
 
@@ -975,7 +981,7 @@ struct rfc822 *make_rfc822(char *filename)/*{{{*/
     result = data_to_rfc822(data, len);
   }
   
-  if (munmap(data, len) < 0) {
+  if (munmap(data, (size_t) len) < 0) {
     perror("Could not munmap");
   }
 
